@@ -8,18 +8,18 @@ use feature 'signatures';
 
 use CXC::Data::Visitor 'visit', 'RESULT_CONTINUE';
 
-my %struct = (
+my %root = (
     fruit => {
         berry  => 'purple',
         apples => [ 'fuji', 'macoun' ],
     } );
 
 visit(
-    \%struct,
+    \%root,
     sub ( $kydx, $vref, @ ) {
         $vref->$* = 'blue' if $kydx eq 'berry';
         return RESULT_CONTINUE;
     } );
 
-say $struct{fruit}{berry}
+say $root{fruit}{berry}
 
