@@ -11,7 +11,7 @@ use feature 'current_sub';
 use experimental 'signatures', 'lexical_subs', 'postderef';
 
 #<<< no tidy
-our $VERSION = '0.10'; # TRIAL
+our $VERSION = '0.11'; # TRIAL
 #>>>
 
 use base 'Exporter::Tiny';
@@ -29,24 +29,27 @@ use constant {
 };
 use constant CYCLE_QR => qr /\A die|continue|truncate \z/x;
 use constant {
-    VISIT_HASH      => 0b001,
-    VISIT_ARRAY     => 0b010,
-    VISIT_CONTAINER => 0b011,
-    VISIT_LEAF      => 0b100,
-    VISIT_ALL       => 0b111,
+    VISIT_HASH      => 0b0001,
+    VISIT_ARRAY     => 0b0010,
+    VISIT_CONTAINER => 0b0011,
+    VISIT_LEAF      => 0b0100,
+    VISIT_ALL       => 0b0111,
+    VISIT_ROOT      => 0b1000,
 };
 use constant {
-    RESULT_NULL              => 0b00000,
-    RESULT_RETURN            => 0b00001,
-    RESULT_CONTINUE          => 0b00010,
-    RESULT_REVISIT_CONTAINER => 0b00100,
-    RESULT_REVISIT_ELEMENT   => 0b01000,
-    RESULT_STOP_DESCENT      => 0b10000,
+    RESULT_NULL              => 0b000000,
+    RESULT_RETURN            => 0b000001,
+    RESULT_CONTINUE          => 0b000010,
+    RESULT_REVISIT_CONTENTS  => 0b000100,
+    RESULT_REVISIT_CONTAINER => 0b000100,    # back compat
+    RESULT_REVISIT_ELEMENT   => 0b001000,
+    RESULT_STOP_DESCENT      => 0b010000,
+    RESULT_REVISIT_ROOT      => 0b100000,
 };
 
 use constant {
-    PASS_VISIT_ELEMENT   => 0,
-    PASS_REVISIT_ELEMENT => 1,
+    PASS_VISIT_ELEMENT   => 0b01,
+    PASS_REVISIT_ELEMENT => 0b10,
 };
 
 our %EXPORT_TAGS = (
@@ -55,14 +58,35 @@ our %EXPORT_TAGS = (
           RESULT_NULL
           RESULT_RETURN
           RESULT_CONTINUE
+          RESULT_REVISIT_ROOT
+          RESULT_REVISIT_CONTENTS
           RESULT_REVISIT_CONTAINER
           RESULT_REVISIT_ELEMENT
           RESULT_STOP_DESCENT
         ),
     ],
-    cycles    => [qw( CYCLE_DIE CYCLE_CONTINUE CYCLE_TRUNCATE )],
-    visits    => [qw( VISIT_ARRAY VISIT_HASH VISIT_CONTAINER VISIT_LEAF VISIT_ALL )],
-    passes    => [qw( PASS_VISIT_ELEMENT PASS_REVISIT_ELEMENT )],
+
+    cycles => [ qw(
+          CYCLE_DIE
+          CYCLE_CONTINUE
+          CYCLE_TRUNCATE
+        ),
+    ],
+
+    visits => [ qw(
+          VISIT_ARRAY
+          VISIT_HASH
+          VISIT_CONTAINER
+          VISIT_LEAF
+          VISIT_ALL
+          VISIT_ROOT
+        ),
+    ],
+    passes => [ qw(
+          PASS_VISIT_ELEMENT
+          PASS_REVISIT_ELEMENT
+        ),
+    ],
     constants => [qw( :results :cycles :visits )],
 );
 
@@ -75,16 +99,20 @@ my sub croak {
 
 
 ## no critic (Subroutines::ProhibitManyArgs  Subroutines::ProhibitExcessComplexity)
-my sub _visit ( $node, $code, $context, $cycle, $visit, $meta ) {
+my sub visit_node ( $node, $code, $context, $cycle, $visit, $meta ) {
+
+    my $path      = $meta->{path};
+    my $ancestors = $meta->{ancestors};
 
-    my $path          = $meta->{path};
-    my $ancestors     = $meta->{ancestors};
-    my $revisit_limit = $meta->{revisit_limit};
     my $sort_key_mode = $meta->{sort_key_mode};
     my $sort_idx_mode = $meta->{sort_idx_mode};
     my $key_sort      = $meta->{key_sort};
     my $idx_sort      = $meta->{idx_sort};
 
+    my $visit_leaf  = !!( $visit & VISIT_LEAF );
+    my $visit_hash  = !!( $visit & VISIT_HASH );
+    my $visit_array = !!( $visit & VISIT_ARRAY );
+
     my $refaddr = refaddr( $node );
     if ( exists $meta->{seen}{$refaddr} ) {
 
@@ -101,13 +129,12 @@ my sub _visit ( $node, $code, $context, $cycle, $visit, $meta ) {
           or croak( __PACKAGE__ . "::visit: unkown cycle parameter value: $lcycle" );
     }
 
-    # after this call to _visit, will have visited all descendents of
+    # after this call to visit_node, will have visited all descendents of
     # $node, so don't need this any longer.
     $meta->{seen}{$refaddr} = ();
     defer { delete $meta->{seen}{$refaddr} }
 
     my %meta = $meta->%*;
-    @meta{ 'visit', 'idx' } = ( 0, -1 );
     $meta{container} = $node;
 
     # deal with bare next in $code body
@@ -118,10 +145,8 @@ my sub _visit ( $node, $code, $context, $cycle, $visit, $meta ) {
     push $ancestors->@*, $node;
     defer { pop $ancestors->@* };
 
-    my $visit_leaf  = !!( $visit & VISIT_LEAF );
-    my $visit_hash  = !!( $visit & VISIT_HASH );
-    my $visit_array = !!( $visit & VISIT_ARRAY );
-
+    my $revisit_limit = $meta->{revisit_limit};
+    @meta{ 'visit', 'idx' } = ( 0, -1 );
 
   SCAN: {
         last unless --$revisit_limit;
@@ -129,7 +154,7 @@ my sub _visit ( $node, $code, $context, $cycle, $visit, $meta ) {
         $meta{visit}++;
         $meta{idx} = -1;
 
-        my $revisit_container = !!0;
+        my $rescan_container = !!0;
 
         my $kydx_arr = do {
 
@@ -169,10 +194,11 @@ my sub _visit ( $node, $code, $context, $cycle, $visit, $meta ) {
                 # immediate rescan if explicitly set to value,
                 # otherwise it will happen after the container is
                 # completely visited
-                redo SCAN if $result == RESULT_REVISIT_CONTAINER;
+                redo SCAN                  if $result == RESULT_REVISIT_CONTENTS;
+                return RESULT_RETURN       if $result == RESULT_RETURN;
+                return RESULT_REVISIT_ROOT if $result == RESULT_REVISIT_ROOT;
 
-                return !!0 if $result == RESULT_RETURN;
-                $revisit_container = $result & RESULT_REVISIT_CONTAINER;
+                $rescan_container = $result & RESULT_REVISIT_CONTENTS;
 
                 next if $result & RESULT_STOP_DESCENT;    # this works for both leaves and containers
 
@@ -186,27 +212,73 @@ my sub _visit ( $node, $code, $context, $cycle, $visit, $meta ) {
 
             my $ref = $vref->$*;
             if ( is_plain_arrayref( $ref ) || is_plain_hashref( $ref ) ) {
-                __SUB__->( $ref, $code, $context, $cycle, $visit, \%meta ) || return !!0;
+                my $result = __SUB__->( $ref, $code, $context, $cycle, $visit, \%meta );
+                return RESULT_RETURN       if $result == RESULT_RETURN;
+                return RESULT_REVISIT_ROOT if $result == RESULT_REVISIT_ROOT;
                 if ( $revisit_element ) {
                     $meta{pass} = PASS_REVISIT_ELEMENT;
-                    my $result = $code->( $kydx, $vref, $context, \%meta );
-                    return !!0 if $result == RESULT_RETURN;
-                    croak( "unknown return value from visit: $result" )
-                      if $result & ~( RESULT_CONTINUE | RESULT_REVISIT_CONTAINER );
-                    $revisit_container |= $result & RESULT_REVISIT_CONTAINER;
+                    $result = $code->( $kydx, $vref, $context, \%meta );
+                    return RESULT_RETURN       if $result == RESULT_RETURN;
+                    return RESULT_REVISIT_ROOT if $result == RESULT_REVISIT_ROOT;
+                    croak( "unexpected return value from visit: $result" )
+                      if $result & ~( RESULT_CONTINUE | RESULT_REVISIT_CONTENTS );
+                    $rescan_container |= $result & RESULT_REVISIT_CONTENTS;
                 }
             }
         }
-        redo SCAN if $revisit_container;
+        redo SCAN if $rescan_container;
     }
     croak( "exceeded limit ($meta{revisit_limit}) on revisiting containers" )
       unless $revisit_limit;
 
+    return RESULT_CONTINUE;
+}
+
+my sub visit_root ( $root, $code, $context, $cycle, $visit, $meta ) {
+
+    my %meta          = $meta->%*;
+    my $revisit_limit = $meta{revisit_limit};
+    $meta{pass} = PASS_VISIT_ELEMENT;
+    @meta{ 'visit', 'idx' } = ( 0, 0 );
+
+  FROOT_LOOP:
+    {
+        $meta{visit}++;
+        last unless --$revisit_limit;
+
+        my $result = $code->( undef, \$root, $context, \%meta );
+
+        redo FROOT_LOOP if $result == RESULT_REVISIT_ROOT;
+
+        return !!0 if $result == RESULT_RETURN;
+        return !!1 if $result == RESULT_STOP_DESCENT;
+
+        my $revisit_element = $result & RESULT_REVISIT_ELEMENT;
+
+        croak( "unknown return value from visit: $result" )
+          if !$revisit_element && !$result & RESULT_CONTINUE;
+
+        my $status = visit_node( $root, $code, $context, $cycle, $visit, \%meta );
+        return !!0 if $status == RESULT_RETURN;
+
+        if ( $revisit_element ) {
+            $meta{pass} = PASS_REVISIT_ELEMENT;
+            $result = $code->( undef, \$root, $context, \%meta );
+            return !!0 if $result == RESULT_RETURN;
+            return !!1 if $result == RESULT_CONTINUE;
+            croak( "unexpected return value while revisiting root: $result" );
+        }
+    }
+    croak( "exceeded limit ($meta{revisit_limit}) while revisiting root" )
+      unless $revisit_limit;
+
     return !!1;
 }
 
+
+
 ## critic (Subroutines::ProhibitManyArgs  Subroutines::ProhibitExcessComplexity)
-sub visit ( $struct, $callback, %opts ) {
+sub visit ( $root, $callback, %opts ) {
 
     is_coderef( $callback )
       or croak( q{parameter 'callback' must be a coderef} );
@@ -262,6 +334,7 @@ sub visit ( $struct, $callback, %opts ) {
 
     my $cycle = delete $opts{cycle} // 'die';
     my $visit = delete $opts{visit} // VISIT_ALL;
+    $visit |= VISIT_ALL if $visit == VISIT_ROOT;
 
     $cycle =~ CYCLE_QR
       or croak( "illegal value for cycle parameter: $cycle" );
@@ -271,7 +344,24 @@ sub visit ( $struct, $callback, %opts ) {
 
     lock_hash( %metadata );
     unlock_value( %metadata, 'container' );
-    my $completed = _visit( $struct, $callback, $context, $cycle, $visit, \%metadata );
+
+    my $completed;
+
+    if ( $visit & VISIT_ROOT ) {
+        $completed = visit_root( $root, $callback, $context, $cycle, $visit, \%metadata );
+    }
+    else {
+        my $revisit_limit = $metadata{revisit_limit};
+        while ( --$revisit_limit ) {
+            last unless --$revisit_limit;
+            $completed = visit_node( $root, $callback, $context, $cycle, $visit, \%metadata );
+            last unless $completed == RESULT_REVISIT_ROOT;
+        }
+        croak( "exceeded limit ($metadata{revisit_limit}) while revisiting root" )
+          unless $revisit_limit;
+        $completed = $completed != RESULT_RETURN;
+    }
+
     unlock_hash( %metadata );
 
     delete $metadata{ancestors};    # should be empty, but just in case,
@@ -305,26 +395,26 @@ CXC::Data::Visitor - Invoke a callback on every element at every level of a data
 
 =head1 VERSION
 
-version 0.10
+version 0.11
 
 =head1 SYNOPSIS
 
  use CXC::Data::Visitor 'visit', 'RESULT_CONTINUE';
  
- my %struct = (
+ my %root = (
      fruit => {
          berry  => 'purple',
          apples => [ 'fuji', 'macoun' ],
      } );
  
  visit(
-     \%struct,
+     \%root,
      sub ( $kydx, $vref, @ ) {
          $vref->$* = 'blue' if $kydx eq 'berry';
          return RESULT_CONTINUE;
      } );
  
- say $struct{fruit}{berry}
+ say $root{fruit}{berry}
 
 results in
 
@@ -333,10 +423,10 @@ results in
 =head1 DESCRIPTION
 
 B<CXC::Data::Visitor::visit> performs a depth-first traversal of a data
-structure, invoking a provided subroutine on elements in the
+structure, invoking a provided callback subroutine on elements in the
 structure.
 
-Here's a partial list of features;
+=head2 Features
 
 =over
 
@@ -375,16 +465,60 @@ Containers that can be reached multiple times without cycling are visited once p
 
 =back
 
+=head2 Overview
+
+C<visit> recursively traverses the container, C<$root>, calling the
+passed subroutine, C<$callback> on each element, C<$element>, which
+is allowed by the L</visit> option.
+
+The traversal is depth-first, e.g. if C<$element> is a
+container, C<$callback> is called on it and then its contents before
+processing C<$element>'s siblings.
+
+Each container's contents are traversed in sorted order.  For hashes,
+this is alphabetical, for arrays, numerical. (This may be
+changed with the L</key_sort> and L</idx_sort> options).
+
+For example, the default traversal order for the structure in the L</SYNOPSIS> is
+
+ +-------------------------+-----------------------+-----+
+ | path                    | value                 | idx |
+ +-------------------------+-----------------------+-----+
+ | $root{fruit}            | \$root{fruit}         | 0   |
+ | $root{fruit}{apples}    | \$root{fruit}{apples} | 0   |
+ | $root{fruit}{apples}[0] | fuji                  | 0   |
+ | $root{fruit}{apples}[1] | macoun                | 1   |
+ | $root{fruit}{berry}     | purple                | 1   |
+ +-------------------------+-----------------------+-----+
+
+Containers that can be reached multiple times without cycling, e.g.
+
+  %hash = ( a => { b => 1 }, );
+  $hash{c} = $hash{a};
+
+are visited once per parent, e.g.
+
+  {a}, {a}{b}
+  {c}, {c}{b}
+
+C<$callback>'s return value indicates how C<visit> should proceed (see
+L</Traversal Directives>).  The simplest directive is to continue
+traversal; additional directives abort the traversal,
+abort descent into a container, revisit the current container
+immediately, revisit a container after its contents are visited,
+and other obscure combinations.
+
 =head1 USAGE
 
 C<visit> has the following signature:
 
-  ( $completed, $context, $metadata ) = visit( $struct, $callback, %options )
+  ( $completed, $context, $metadata ) = visit( $root, $callback, %options )
 
-The two mandatory arguments are C<$struct>, a reference to either a
-hash or an array, and C<$callback>, a reference to a subroutine.
-
-C<visit> returns the following:
+The two mandatory arguments are C<$root>, a reference to either a
+hash or an array, and C<$callback>, a reference to a subroutine which
+will be invoked on visited elements. By default C<$callback> is invoked on
+C<$root>'s elements, not on C<$root> itself; use the L</VISIT_ROOT>
+flag change this.
 
 =over
 
@@ -409,19 +543,20 @@ C<visit> may be passed the following options:
 
 =over
 
-=item B<context> I<optional>
+=item context I<optional>
 
 Arbitrary data to be passed to L</$callback> via the C<$context>
 argument. Use it for whatever you'd like.  If not specified, a hash
 will be created.
 
-=item B<cycle> => I<constant|coderef>
+=item cycle => I<constant|coderef>
 
-How cycles within C<$struct> should be handled.  See L</Cycles>.
+How cycles within C<$root> should be handled.  See L</Cycles>.
 
-=item I<visit> => I<constant>
+=item visit => I<constant>
 
-filter out elements passed to C<$callback>.  See L</Element Filtering>
+Specify elements (by type) which will be passed to C<$callback>.  See
+L</Element Filters>
 
 =item key_sort => I<boolean> | C<$coderef>
 
@@ -454,32 +589,31 @@ than, equal to, or greater than that of the second argument.
 
 =item revisit_limit
 
-If L</$callback> returns B<RESULT_REVISIT_CONTAINER> the element's
-parent container is re-scanned for its elements and revisited.  To
-avoid an inadvertent infinite loop, an exception is thrown if the
-parent container is revisited more than this number of times.  It
-defaults to 10; Set it to C<0> to indicate no limit.
+A container may be scanned multiple times during a visit to it.
+This sets the maximum number of times the container is re-scanned
+during a visit before C<visit> throws an exception to avoid infinite
+loops.  This limit also applies to L</RESULT_REVISIT_ROOT>.
+
+The defaults is C<10>. Set it to C<0> to indicate no limit.
 
 =back
 
 =head2 Callback
 
-C<visit> invokes C<$callback> on selected elements of C<$struct> (see
+C<visit> invokes C<$callback> on selected elements of C<$root> (see
 L</Element Filters>). C<$callback> is invoked as
 
   $directive = $callback->( $kydx, $vref, $context, \%metadata );
 
-The returned value, C<$directive>, informs C<visit> how it should
-proceed. Its values are described in L</Traversal Directives>
-
 The arguments passed to C<$callback> are:
 
 =over
 
 =item B<$kydx>
 
-The location (key or index) of the element in its parent container
-(hash or array).
+The location (key or index) of the element in its parent
+container. This will be undefined when C<$callback> is invoked on
+C<$root> (see L</VISIT_ROOT>).
 
 =item B<$vref>
 
@@ -500,111 +634,98 @@ See L</Metadata>
 
 =back
 
-=head2 Traversal Order
-
-C<$struct> is traversed in depth-first order.  At each level elements
-are traversed in sorted order.  For hashes, this is alphabetical, for
-arrays, numerical. The defaults may be changed with the L</key_sort>
-and L</idx_sort> options.
-
-The default traversal order for the structure in the L</SYNOPSIS> is
-
- +---------------------------+-------------------------+-----+
- | path                      | value                   | idx |
- +---------------------------+-------------------------+-----+
- | $struct{fruit}            | \$struct{fruit}         | 0   |
- | $struct{fruit}{apples}    | \$struct{fruit}{apples} | 0   |
- | $struct{fruit}{apples}[0] | fuji                    | 0   |
- | $struct{fruit}{apples}[1] | macoun                  | 1   |
- | $struct{fruit}{berry}     | purple                  | 1   |
- +---------------------------+-------------------------+-----+
-
-Containers that can be reached multiple times without cycling, e.g.
-
-  %hash = ( a => { b => 1 }, );
-  $hash{c} = $hash{a};
-
-are visited once per parent, e.g.
-
-  {a}, {a}{b}, {a}{b}[0]
-  {c}, {c}{b}, {c}{b}[0]
-
 =head2 Traversal Directives
 
 L</$callback> must return a constant (see L</EXPORTS>)
-indicating what C<visit> should do next.
+indicating what C<visit> should do next.  Not all constants
+are allowed in all contexts in which C<$callback> is invoked;
+see L</Calling Contexts and Allowed Traversal Directives>.
+
+=head3 Single Directives
 
 =over
 
 =item RESULT_CONTINUE
 
-continue traversing as normal.
+Visit the next element in the parent container.
 
- +---------------------------+-------------------------+-----+
- | path                      | value                   | idx |
- +---------------------------+-------------------------+-----+
- | $struct{fruit}            | \$struct{fruit}         | 0   |
- | $struct{fruit}{apples}    | \$struct{fruit}{apples} | 0   |
- | $struct{fruit}{apples}[0] | fuji                    | 0   |
- | $struct{fruit}{apples}[1] | macoun                  | 1   |
- | $struct{fruit}{berry}     | purple                  | 1   |
- +---------------------------+-------------------------+-----+
+ +-------------------------+-----------------------+-----+
+ | path                    | value                 | idx |
+ +-------------------------+-----------------------+-----+
+ | $root{fruit}            | \$root{fruit}         | 0   |
+ | $root{fruit}{apples}    | \$root{fruit}{apples} | 0   |
+ | $root{fruit}{apples}[0] | fuji                  | 0   |
+ | $root{fruit}{apples}[1] | macoun                | 1   |
+ | $root{fruit}{berry}     | purple                | 1   |
+ +-------------------------+-----------------------+-----+
 
 =item RESULT_RETURN
 
-return immediately to the caller of C<visit>.
+Return immediately to the caller of C<visit>.
 
 =item RESULT_STOP_DESCENT
 
-If the current element is a hash or array, do not visit its contents.
+If the current element is a hash or array, do not visit its contents,
+and visit the next element in the parent container.
 
-For non-container elements, this is equivalent to L</RESULT_CONTINUE>.
+If the element is not a container, the next element in the container
+will be visited (just as with L</RESULT_CONTINUE>).
 
 For example, If C<RESULT_STOP_DESCENT> is returned when
-C<$struct{fruit}{apples}> is traversed, the traversal would look like
+C<$root{fruit}{apples}> is traversed, the traversal would look like
 this:
 
- +------------------------+-------------------------+-----+
- | path                   | value                   | idx |
- +------------------------+-------------------------+-----+
- | $struct{fruit}         | \$struct{fruit}         | 0   |
- | $struct{fruit}{apples} | \$struct{fruit}{apples} | 0   |
- | $struct{fruit}{berry}  | purple                  | 1   |
- +------------------------+-------------------------+-----+
+ +----------------------+-----------------------+-----+
+ | path                 | value                 | idx |
+ +----------------------+-----------------------+-----+
+ | $root{fruit}         | \$root{fruit}         | 0   |
+ | $root{fruit}{apples} | \$root{fruit}{apples} | 0   |
+ | $root{fruit}{berry}  | purple                | 1   |
+ +----------------------+-----------------------+-----+
 
-=item RESULT_REVISIT_CONTAINER
+=item RESULT_REVISIT_CONTENTS
 
-Immediately stop processing the current container, and start traversing
-it again.
+Do not visit the next element in the parent container. restart with
+the first element in the container.  The order of elements is
+determined when the container is visited, so starts within a visit
+will have the same order.
 
-For example, if C<RESULT_REVISIT_CONTAINER> is returned the
-first time C<$struct{fruit}{apples}[0]> is traversed, the
+For example, if C<RESULT_REVISIT_CONTENTS> is returned the
+first time C<$root{fruit}{apples}[0]> is traversed, the
 traversal would look like this:
 
- +---------------------------+-------------------------+-----+-------+
- | path                      | value                   | idx | visit |
- +---------------------------+-------------------------+-----+-------+
- | $struct{fruit}            | \$struct{fruit}         | 0   | 1     |
- | $struct{fruit}{apples}    | \$struct{fruit}{apples} | 0   | 1     |
- | $struct{fruit}{apples}[0] | fuji                    | 0   | 1     |
- | $struct{fruit}{apples}[0] | fuji                    | 0   | 2     |
- | $struct{fruit}{apples}[1] | macoun                  | 1   | 2     |
- | $struct{fruit}{berry}     | purple                  | 1   | 1     |
- +---------------------------+-------------------------+-----+-------+
+ +-------------------------+-----------------------+-----+-------+
+ | path                    | value                 | idx | visit |
+ +-------------------------+-----------------------+-----+-------+
+ | $root{fruit}            | \$root{fruit}         | 0   | 1     |
+ | $root{fruit}{apples}    | \$root{fruit}{apples} | 0   | 1     |
+ | $root{fruit}{apples}[0] | fuji                  | 0   | 1     |
+ | $root{fruit}{apples}[0] | fuji                  | 0   | 2     |
+ | $root{fruit}{apples}[1] | macoun                | 1   | 2     |
+ | $root{fruit}{berry}     | purple                | 1   | 1     |
+ +-------------------------+-----------------------+-----+-------+
 
 To avoid inadvertent infinite loops, the number of revisits
 during a traversal of a container is limited (see L</revisit_limit>).
 Containers with multiple parents are traversed once per parent; The
 limit is reset for each traversal.
 
+=item RESULT_REVISIT_ROOT
+
+Stop processing and re-start at C<$root>.
+To avoid inadvertent infinite loops, the number of revisits
+is limited (see L</revisit_limit>).
+
 =item RESULT_REVISIT_ELEMENT
 
-If the element is a container, L</$callback> will be invoked with it
-after its contents are traversed.
+If the current element is not a container, the next element in the
+container will be visited (just as with L</RESULT_CONTINUE>).
 
-If the element is not a container, the result is the same as L</RESULT_CONTINUE>.
+If the current element is a container, its contents will be visited,
+and L</$callback> will be invoked on it again afterwards.
 
-During the initial visit
+During the call to C<$callback> on the container prior to visiting
+its contents,
 
   $metadata->{pass} & PASS_VISIT_ELEMENT
 
@@ -612,79 +733,169 @@ will be true.  During the followup visit
 
   $metadata->{pass} & PASS_REVISIT_ELEMENT
 
-will be true. During this visit, L</$callback> must return one of the following:
-
-  RESULT_RETURN
-  RESULT_CONTINUE
-  RESULT_CONTINUE | RESULT_REVISIT_CONTAINER
-
-otherwise an exception will be thrown.
+will be true.
 
 For example, If C<RESULT_REVISIT_ELEMENT> is returned when
-C<$struct{fruit}{apples}> is traversed, the traversal would look like
+C<$root{fruit}{apples}> is traversed, the traversal would look like
 this:
 
- +---------------------------+-------------------------+-----+----------------------+
- | path                      | value                   | idx | pass                 |
- +---------------------------+-------------------------+-----+----------------------+
- | $struct{fruit}            | \$struct{fruit}         | 0   | PASS_VISIT_ELEMENT   |
- | $struct{fruit}{apples}    | \$struct{fruit}{apples} | 0   | PASS_VISIT_ELEMENT   |
- | $struct{fruit}{apples}[0] | fuji                    | 0   | PASS_VISIT_ELEMENT   |
- | $struct{fruit}{apples}[1] | macoun                  | 1   | PASS_VISIT_ELEMENT   |
- | $struct{fruit}{apples}    | \$struct{fruit}{apples} | 0   | PASS_REVISIT_ELEMENT |
- | $struct{fruit}{berry}     | purple                  | 1   | PASS_VISIT_ELEMENT   |
- +---------------------------+-------------------------+-----+----------------------+
+ +-------------------------+-----------------------+-----+----------------------+
+ | path                    | value                 | idx | pass                 |
+ +-------------------------+-----------------------+-----+----------------------+
+ | $root{fruit}            | \$root{fruit}         | 0   | PASS_VISIT_ELEMENT   |
+ | $root{fruit}{apples}    | \$root{fruit}{apples} | 0   | PASS_VISIT_ELEMENT   |
+ | $root{fruit}{apples}[0] | fuji                  | 0   | PASS_VISIT_ELEMENT   |
+ | $root{fruit}{apples}[1] | macoun                | 1   | PASS_VISIT_ELEMENT   |
+ | $root{fruit}{apples}    | \$root{fruit}{apples} | 0   | PASS_REVISIT_ELEMENT |
+ | $root{fruit}{berry}     | purple                | 1   | PASS_VISIT_ELEMENT   |
+ +-------------------------+-----------------------+-----+----------------------+
 
 =back
 
-Some directives can be mixed with L</RESULT_REVISIT_CONTAINER> and
-L</RESULT_REVISIT_ELEMENT> by supplying a binary with them.
+=head3 Mixed Directives
+
+Some directives can be mixed with L</RESULT_REVISIT_CONTENTS> and
+L</RESULT_REVISIT_ELEMENT> by performing a binary OR with them.
 
 =over
 
-=item RESULT_STOP_DESCENT | RESULT_REVISIT_CONTAINER
+=item RESULT_STOP_DESCENT | RESULT_REVISIT_CONTENTS
 
-Modify the next processing step as performed by L</RESULT_STOP_DESCENT>
+If the current element is not a container, the next element in the
+container will be visited (just as with L</RESULT_CONTINUE>).
 
-Additionally, revisit the container after the rest of the container
-has been visited.
+If the current element is a hash or array, do not visit its contents,
+and continue with the next element in the parent container.  For
+non-container elements, continue with the next element in the parent
+container.
 
-For example, if C<RESULT_STOP_DESCENT | RESULT_REVISIT_CONTAINER> is
-returned when C<$struct{fruit}{apples}> is traversed when
+After all of the container's contents have been visited, start
+again with the first element in the container.
+
+For example, if C<RESULT_STOP_DESCENT | RESULT_REVISIT_CONTENTS> is
+returned when C<$root{fruit}{apples}> is traversed when
 C<$metadata->{visit} ==1>, the traversal would look like
 
- +------------------------+-------------------------+-----+-------+
- | path                   | value                   | idx | visit |
- +------------------------+-------------------------+-----+-------+
- | $struct{fruit}         | \$struct{fruit}         | 0   | 1     |
- | $struct{fruit}{apples} | \$struct{fruit}{apples} | 0   | 1     |
- | $struct{fruit}{berry}  | purple                  | 1   | 1     |
- | $struct{fruit}{apples} | \$struct{fruit}{apples} | 0   | 2     |
- | $struct{fruit}{berry}  | purple                  | 1   | 2     |
- +------------------------+-------------------------+-----+-------+
+ +----------------------+-----------------------+-----+-------+
+ | path                 | value                 | idx | visit |
+ +----------------------+-----------------------+-----+-------+
+ | $root{fruit}         | \$root{fruit}         | 0   | 1     |
+ | $root{fruit}{apples} | \$root{fruit}{apples} | 0   | 1     |
+ | $root{fruit}{berry}  | purple                | 1   | 1     |
+ | $root{fruit}{apples} | \$root{fruit}{apples} | 0   | 2     |
+ | $root{fruit}{berry}  | purple                | 1   | 2     |
+ +----------------------+-----------------------+-----+-------+
 
-=item RESULT_CONTINUE | RESULT_REVISIT_CONTAINER
+=item RESULT_CONTINUE | RESULT_REVISIT_CONTENTS
 
-Modify the next processing step as performed by L</RESULT_CONTINUE>
+Visit the remaining elements in the parent container, then start again
+with the first element in the container.
 
-Additionally, revisit the container after the rest of the container
-has been visited.
-
-For example, if C<RESULT_CONTINUE | RESULT_REVISIT_CONTAINER> is
+For example, if C<RESULT_CONTINUE | RESULT_REVISIT_CONTENTS> is
 returned when C<$callback> is first passed
-C<$struct{fruit}{apples}[0]>, the traversal would look like
+C<$root{fruit}{apples}[0]>, the traversal would look like
 
- +---------------------------+-------------------------+-----+-------+
- | path                      | value                   | idx | visit |
- +---------------------------+-------------------------+-----+-------+
- | $struct{fruit}            | \$struct{fruit}         | 0   | 1     |
- | $struct{fruit}{apples}    | \$struct{fruit}{apples} | 0   | 1     |
- | $struct{fruit}{apples}[0] | fuji                    | 0   | 1     |
- | $struct{fruit}{apples}[1] | macoun                  | 1   | 1     |
- | $struct{fruit}{apples}[0] | fuji                    | 0   | 2     |
- | $struct{fruit}{apples}[1] | macoun                  | 1   | 2     |
- | $struct{fruit}{berry}     | purple                  | 1   | 1     |
- +---------------------------+-------------------------+-----+-------+
+ +-------------------------+-----------------------+-----+-------+
+ | path                    | value                 | idx | visit |
+ +-------------------------+-----------------------+-----+-------+
+ | $root{fruit}            | \$root{fruit}         | 0   | 1     |
+ | $root{fruit}{apples}    | \$root{fruit}{apples} | 0   | 1     |
+ | $root{fruit}{apples}[0] | fuji                  | 0   | 1     |
+ | $root{fruit}{apples}[1] | macoun                | 1   | 1     |
+ | $root{fruit}{apples}[0] | fuji                  | 0   | 2     |
+ | $root{fruit}{apples}[1] | macoun                | 1   | 2     |
+ | $root{fruit}{berry}     | purple                | 1   | 1     |
+ +-------------------------+-----------------------+-----+-------+
+
+=back
+
+=head2 Calling Contexts and Allowed Traversal Directives
+
+C<$callback>'s allowed return value depends upon the context it is
+called in.  C<$callback> may be called on an element multiple times
+during different stages of traversal.
+
+=head3 When invoked on an element during a scan of its parent container
+
+=over
+
+=item *
+
+The C<pass> metadata attribute is set to C<PASS_VISIT_ELEMENT>
+
+=item *
+
+C<$callback> must return one of
+
+  RESULT_REVISIT_CONTENTS
+  RESULT_RETURN
+  RESULT_CONTINUE
+
+  RESULT_CONTINUE | RESULT_REVISIT_CONTENTS
+  RESULT_STOP_DESCENT | RESULT_REVISIT_CONTENTS
+  RESULT_CONTINUE | RESULT_REVISIT_ELEMENT
+
+=back
+
+=head3 When invoked on a container immediately after its contents have been visited
+
+See L</RESULT_REVISIT_ELEMENT>.
+
+=over
+
+=item *
+
+The C<pass> metadata attribute is set to C<PASS_REVISIT_ELEMENT>
+
+=item *
+
+C<$callback> must return one of
+
+  RESULT_RETURN
+  RESULT_CONTINUE
+  RESULT_REVISIT_CONTENTS
+  RESULT_CONTINUE | RESULT_REVISIT_CONTENTS
+
+=back
+
+=head3 When invoked on C<$root> before its contents have been visited
+
+See L</VISIT_ROOT>.
+
+=over
+
+=item *
+
+The C<pass> metadata attribute is set to C<PASS_VISIT_ELEMENT>
+
+=item *
+
+C<$callback> must return one of
+
+  RESULT_CONTINUE
+  RESULT_CONTINUE | RESULT_REVISIT_ELEMENT
+  RESULT_RETURN
+  RESULT_REVISIT_ROOT
+  RESULT_STOP_DESCENT
+
+=back
+
+=head3 When invoked on the C<$root> immediately after its elements have been visited
+
+See L</VISIT_ROOT> and L</RETURN_REVISIT_ELEMENT>.
+
+=over
+
+=item *
+
+The C<pass> metadata attribute is set to C<PASS_REVISIT_ELEMENT>
+
+=item *
+
+C<$callback> must return one of
+
+  RESULT_RETURN
+  RESULT_CONTINUE
 
 =back
 
@@ -705,7 +916,7 @@ A reference to the hash or array which contains the element being visited.
 =item B<path>
 
 An array which contains the path (keys and indices) used to arrive
-at the current element from B<$struct>.
+at the current element from B<$root>.
 
 =item B<ancestors>
 
@@ -719,9 +930,12 @@ See L</RESULT_REVISIT_ELEMENT>.
 
 =item B<visit>
 
-A unary-based counter indicating the number of times a container has
-been scanned in a single visit.  This will be greater than C<1> if the
-L</RESULT_REVISIT_CONTAINER> directive has been applied.
+A unary-based counter indicating the number of times the element's
+container has been scanned and its contents processed in a single
+visit.  This will be greater than C<1> if the
+L</RESULT_REVISIT_CONTENTS> directive has been applied. It is I<not>
+the number of times that the element has been visited, as scans may be
+interrupted and restarted.
 
 =item B<idx>
 
@@ -733,7 +947,10 @@ L</key_sort> and L</idx_sort>.
 
 =head2 Element Filters
 
-The parts of the structure that will trigger a callback.
+The parts of the structure that will trigger a callback.  Note that
+by default the passed top level structure, C<$root> is I<not>
+passed to the callback.  See L</VISIT_ROOT>.
+
 See L</EXPORTS> to import the constants.
 
 =over
@@ -743,12 +960,12 @@ See L</EXPORTS> to import the constants.
 Invoke L</$callback> on containers (either hashes or arrays).  For
 example, the elements in the following structure
 
-  $struct = { a => { b => 1, c => [ 2, 3 ] } }
+  $root = { a => { b => 1, c => [ 2, 3 ] } }
 
 passed to L</$callback> are:
 
-  a => {...}  # $struct->{a}
-  c => [...]  # $struct->{c}
+  a => {...}  # $root->{a}
+  c => [...]  # $root->{c}
 
 =item VISIT_ARRAY
 
@@ -761,17 +978,28 @@ Only visit containers of the given type.
 Invoke L</$callback> on terminal (leaf) elements.  For example, the
 elements in the following structure
 
-  $struct = { a => { b => 1, c => [ 2, 3 ] } }
+  $root = { a => { b => 1, c => [ 2, 3 ] } }
 
 passed to L</$callback> are:
 
-  b => 1  # $struct->{a}{b}
-  0 => 2  # $struct->{a}{c}[0]
-  1 => 3  # $struct->{a}{c}[1]
+  b => 1  # $root->{a}{b}
+  0 => 2  # $root->{a}{c}[0]
+  1 => 3  # $root->{a}{c}[1]
 
 =item VISIT_ALL
 
-Invoke L</$callback> on all elements.  This is the default.
+Invoke L</$callback> on all elements except for C<$root>.  This is the default.
+
+=item VISIT_ROOT
+
+Pass C<$root> to C<$callback>. To filter on one of the other values, pass
+a binary OR of L</VISIT_ROOT> and the other filter, e.g.
+
+  VISIT_ROOT | VISIT_LEAF
+
+Specifying L</VISIT_ROOT> on its own is equivalent to
+
+  VISIT_ROOT | VISIT_ALL
 
 =back
 
@@ -812,21 +1040,16 @@ indicating what should be done.
 This module uses L<Exporter::Tiny>, which provides enhanced import
 utilities.
 
-The following symbols may be exported:
+=head2 Subroutines
+
+The following subroutine may be imported:
 
  visit
 
- VISIT_CONTAINER VISIT_LEAF VISIT_ALL
+=head3 Constants
 
- CYCLE_DIE CYCLE_CONTINUE CYCLE_TRUNCATE
-
- RESULT_RETURN RESULT_CONTINUE
- RESULT_REVISIT_CONTAINER RESULT_REVISIT_ELEMENT
- RESULT_STOP_DESCENT
-
- PASS_VISIT_ELEMENT PASS_REVISIT_ELEMENT
-
-The available tags and their respective imported symbols are:
+Constants may be imported individually or as groups via tags.  The
+available tags and their respective imported symbols are:
 
 =over
 
@@ -837,7 +1060,10 @@ Import all symbols.
 =item B<results>
 
  RESULT_RETURN RESULT_CONTINUE
- RESULT_REVISIT_CONTAINER RESULT_REVISIT_ELEMENT
+ RESULT_REVISIT_CONTENTS
+ RESULT_REVISIT_CONTAINER  # deprecated alias for RESULT_REVISIT_CONTENTS
+ RESULT_REVISIT_ROOT
+ RESULT_REVISIT_ELEMENT
  RESULT_STOP_DESCENT
 
 =item B<cycles>
@@ -858,6 +1084,16 @@ Import tags C<results>, C<cycles>, C<visits>.
 
 =back
 
+=head1 DEPRECATED CONSTRUCTS
+
+=over
+
+=item *
+
+B<RESULT_REVISIT_CONTAINER> is a deprecated alias for L</RESULT_REVISIT_CONTENTS>.
+
+=back
+
 =head1 SUPPORT
 
 =head2 Bugs
@@ -868,11 +1104,11 @@ Please report any bugs or feature requests to bug-cxc-data-visitor@rt.cpan.org
 
 Source is available at
 
-  https://gitlab.com/djerius/cxc-data-visitor
+  https://codeberg.org/CXC-Optics/p5-CXC-Data-Visitor
 
 and may be cloned from
 
-  https://gitlab.com/djerius/cxc-data-visitor.git
+  https://codeberg.org/CXC-Optics/p5-CXC-Data-Visitor.git
 
 =head1 SEE ALSO
 
